import pytest

from faststream import TestApp
from tests.marks import (
    require_aiokafka,
    require_aiopika,
    require_confluent,
    require_nats,
    require_redis,
)


@pytest.mark.asyncio
@require_aiokafka
async def test_custom_global_context_kafka():
    from docs.docs_src.getting_started.context.kafka.custom_global_context import (
        app,
        broker,
        handle,
    )
    from faststream.kafka import TestKafkaBroker

    async with TestKafkaBroker(broker) as br, TestApp(app):
        await br.publish("Hi!", "test-topic")

        handle.mock.assert_called_once_with("Hi!")


@pytest.mark.asyncio
@require_confluent
async def test_custom_global_context_confluent():
    from docs.docs_src.getting_started.context.confluent.custom_global_context import (
        app,
        broker,
        handle,
    )
    from faststream.confluent import TestKafkaBroker as TestConfluentKafkaBroker

    async with TestConfluentKafkaBroker(broker) as br, TestApp(app):
        await br.publish("Hi!", "test-topic")

        handle.mock.assert_called_once_with("Hi!")


@pytest.mark.asyncio
@require_aiopika
async def test_custom_global_context_rabbit():
    from docs.docs_src.getting_started.context.rabbit.custom_global_context import (
        app,
        broker,
        handle,
    )
    from faststream.rabbit import TestRabbitBroker

    async with TestRabbitBroker(broker) as br, TestApp(app):
        await br.publish("Hi!", "test-queue")

        handle.mock.assert_called_once_with("Hi!")


@pytest.mark.asyncio
@require_nats
async def test_custom_global_context_nats():
    from docs.docs_src.getting_started.context.nats.custom_global_context import (
        app,
        broker,
        handle,
    )
    from faststream.nats import TestNatsBroker

    async with TestNatsBroker(broker) as br, TestApp(app):
        await br.publish("Hi!", "test-subject")

        handle.mock.assert_called_once_with("Hi!")


@pytest.mark.asyncio
@require_redis
async def test_custom_global_context_redis():
    from docs.docs_src.getting_started.context.redis.custom_global_context import (
        app,
        broker,
        handle,
    )
    from faststream.redis import TestRedisBroker

    async with TestRedisBroker(broker) as br, TestApp(app):
        await br.publish("Hi!", "test-channel")

        handle.mock.assert_called_once_with("Hi!")
